#include <mkl.h>
#include <stdio.h>
#include <stdlib.h>
#include <sys/time.h>
#include <math.h>
#include <string.h>

#include "utils.h"
#include "eigen.h"

int main(int argc, char** argv)
{
    int n, i;
    double *matrix;
    double *e; /* eigenvalues */
    float et;
    struct timeval t0, t1;
    char fname[128];

    char *inputmatrix = NULL;

    if (argc != 2)
    {
        printf("Usage: %s <matrix file>\n", argv[0]);
	exit(1);
    }
    
    inputmatrix = argv[1];

    /* load input matrix */
    printf("loading matrix....\n");
    matrix = load_matrix(inputmatrix, &n);
    //plot_matrix("original.pgm", matrix, n);
    printf("%ix%i matrix loaded\n", n, n);

    /* allocate memory for e */
    if ((e = (double *)malloc(n * sizeof(double))) == NULL)
    {
	printf("ERROR: malloc failed for vector size %i\n", n);
	exit(1);
    }

    /* call compute subroutine */
    printf("Computing eigen...\n");
    
    gettimeofday(&t0, NULL);
    
    eigen(matrix, n, e); 
    
    gettimeofday(&t1, NULL);
    
    printf("Done in %.2f sec\n", TDIFF(t1, t0));

    sprintf(fname, "eigenvectors.dat\0");
    save_submatrix(fname, matrix, n, n);
    
    sprintf(fname, "eigenvalues.dat\0");
    save_vector(fname, e, n);
    
    //plot_matrix("eigenvectors.pgm", matrix, n);

    free(matrix);
    free(e);
}

