#include <stdio.h>
#include <stdlib.h>
#include <math.h>

#include "utils.h"

/* figures out size of a square matrix and loads it to memory */
double* load_matrix(char *inputfile, int *size)
{
    int i, n, n2;
    float tmp;
    FILE *file;
    double* matrix;
    
    if ((file = fopen(inputfile, "r")) == NULL)
    {
	printf("ERROR: Failed to open file %s for reading\n", inputfile);
	exit(1);
    }

    /* figure out size of the matrix */
    n2 = 0;
    while (fscanf(file, "%f\n", &tmp) == 1) n2++;
    
    n = sqrtf(n2);
    if (n * n != n2)
    {
        printf("ERROR: Found %i elements. Input matrix is not square\n", n2);
	exit(1);
    }
    
    *size = n;
    if ((matrix = malloc(n2 * sizeof(double))) == NULL)
    {
	printf("ERROR: malloc failed for matrix size %ix%i\n", n, n);
	exit(1);
    }
    
    /* read the file */
    fseek(file, 0, SEEK_SET);

    for (i = 0; i < n2; i++)
    {
	fscanf(file, "%f\n", &tmp);
	matrix[i] = tmp;
    }
    
    fclose(file);
    
#if 0
    /* display a sample */
    printf("4x4 matrix sample\n");
    int j;
    for (i =0;i < 4;i++) 
    {
        for (j =0; j < 4; j++) 
        { 
            printf("%f ", matrix[i*N+j]);
        }
        printf("\n");
    }
#endif

    return matrix;
}

/* saves a square matrix to file */
void save_matrix(char *outputfile, double *matrix, int n)
{
    int i;
    FILE *file;
    
    if ((file = fopen(outputfile, "w")) == NULL)
    {
	printf("ERROR: Failed to open file %s for writing\n", outputfile);
	return;
    }
    
    for (i = 0; i < n*n; i++)
    {
	fprintf(file, "%f\n", matrix[i]);
    }
    
    fclose(file);
}

/* save a submatrix to file */
void save_submatrix(char *outputfile, double *matrix, int N, int n)
{
    int i, j;
    FILE *file;
    
    if ((file = fopen(outputfile, "w")) == NULL)
    {
	printf("ERROR: Failed to open file %s for writing\n", outputfile);
	return;
    }
    
    for (i = 0; i < n; i++)
    {    
        for (j = 0; j < n; j++)
        {
	    fprintf(file, "%f ", matrix[i*N+j]);
        }
        fprintf(file, "\n");
    }
    
    fclose(file);
}


/* saves a vector to file */
void save_vector(char *outputfile, double *vector, int n)
{
    int i;
    FILE *file;
    
    if ((file = fopen(outputfile, "w")) == NULL)
    {
	printf("ERROR: Failed to open file %s for writing\n", outputfile);
	return;
    }
    
    for (i = 0; i < n; i++)
    {
	fprintf(file, "%f\n", vector[i]);
    }
    
    fclose(file);
}

/* produces a pgm image of a matrix with zero elements set to 0 and the rest set to 1 */
void plot_matrix(char *name, double *matrix, int n)
{
    FILE *f;
    int i, j;
    long int addr=0;

    if ((f = fopen(name, "w")) == NULL)
        return;

    fprintf(f, "P2\n%i %i\n1\n", n, n);

    for (i = 0; i < n; i++)
    {
        for (j = 0; j < n; j++)
	{
	    double val = (matrix[addr] < 0.0) ? -matrix[addr] : matrix[addr];
	    fprintf(f, "%i ", (val < 1e-6) ? 0 : 1);
	    addr++;
	}
	
	fprintf(f, "\n");
    }
    
    fclose(f);
}
